from .Criterion import Criterion
from .focal import FocalLoss
from .tversky import TverskyLoss
from torch.nn import MSELoss, SmoothL1Loss, L1Loss


def reg_loss(config):
    loss_functions = []
    if isinstance(config.criterion_reg, str):
        if config.criterion_reg == 'MSELoss':
            loss_functions.append(
                MSELoss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction)
            )
        elif config.criterion_reg == 'L1Loss':
            loss_functions.append(
                L1Loss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction)
            )
        elif config.criterion_reg == 'SmoothL1Loss':
            loss_functions.append(
                SmoothL1Loss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction, beta=config.smoothl1_beta)
            )

    elif isinstance(config.criterion_reg, list):
        for i, _criterion in enumerate(config.criterion_reg):
            if _criterion == 'MSELoss':
                loss_functions.append(
                    MSELoss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction[i])
                )
            elif _criterion == 'L1Loss':
                loss_functions.append(
                    L1Loss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction[i])
                )
            elif _criterion == 'SmoothL1Loss':
                loss_functions.append(
                    SmoothL1Loss(size_average=config.size_averge, reduce=config.reduce, reduction=config.loss_reduction[i], beta=config.smoothl1_beta)
                )
    else:
        raise Exception('Unexpected criterion {}'.format(config.criterion_reg))
    return Criterion(loss_functions, config.criterion_weight_reg, config.loss_threshold)    


def seg_loss(class_weight, config):
    loss_functions = []
    if isinstance(config.criterion_seg, str):
        if config.criterion_seg == 'FocalLoss':
            if config.class_weights_file:
                focal_alpha = class_weight
            else:
                focal_alpha = config.loss_focal_alpha
            loss_functions.append(
                FocalLoss(gamma=config.loss_focal_gamma, alpha=focal_alpha, ignore_index=config.loss_ignore_index, reduction=config.loss_reduction)
            )
        elif config.criterion_seg == 'TverskyLoss':
            loss_functions.append(
                TverskyLoss(alpha=config.loss_tversky_gamma, beta=config.loss_tversky_beta)
            )
    elif isinstance(config.criterion_seg, list):
        for i, _criterion in enumerate(config.criterion_seg):
            if _criterion == 'FocalLoss':
                if config.class_weights_file:
                    focal_alpha = class_weight
                else:
                    focal_alpha = config.loss_focal_alpha
                loss_functions.append(
                    FocalLoss(gamma=config.loss_focal_gamma, reduction=config.loss_reduction[i])
                )
            elif _criterion == 'TverskyLoss':
                loss_functions.append(
                    TverskyLoss(alpha=config.loss_tversky_gamma, beta=config.loss_tversky_beta)
                )
    else:
        raise Exception('Unexpected criterion {}'.format(config.criterion_seg))
    return Criterion(loss_functions, config.criterion_weight_seg)


def generate_criterion(class_weight, config):

    if config.mode == 'segmentation':
        return seg_loss(class_weight, config)

    elif config.mode == 'regression':
        return reg_loss(config)

    else:
        raise Exception(f'Unexpected mode {config.mode}')