import random
import rasterio
import numpy as np
import torch
import torch.nn as nn

class Datautils():

    def load_filelist(self, list_file):
        file_list = []
        with open(list_file, 'r') as f:
            for line in f:
                file_list.append(line.strip())
        print('DataLoader will load {} files from {}'.format(len(file_list), list_file))
        return file_list


    def random_sublist(self, src, num=0, random_seed=42):
        if num <= 0 or num > len(src):
            return src, src
        dst = src.copy()
        random.seed(random_seed)
        random.shuffle(dst)
        train_list = dst[:num]
        val_list = dst[num:]
        return train_list, val_list


    def read_raster(self, filename):
        try:
            with rasterio.open(filename, 'r') as src:
                data = src.read()
        except rasterio.errors.RasterioIOError():
            print(f'Cannot open filename: {filename}')
            return None 
        return data.transpose(1,2,0), src.meta


    def img_norm(self, im, axis=(0,1), c=1e-8, nodata_val=None):
        if nodata_val is not None and np.sum(im == nodata_val) > 0:
            im = im.astype(np.float32)
            im[np.any(im == nodata_val, axis=2), :] = np.nan 
            return (im - np.nanmean(im, axis)) / (np.nanstd(im, axis) + c)
        else:
            return (im - im.mean(axis)) / (im.std(axis) + c)


    def gt_max_pooling(self, gt, kernel_size=3, stride=1, padding=1):
        gt = torch.Tensor(gt[None,...])
        m = nn.MaxPool2d(kernel_size, stride=stride, padding=padding)
        return np.array(m(gt)).squeeze()
