def generate_model(cfg):
    '''
    models: str
        Unet, Unet++, MAnet, Linknet, FPN, PSPNet, PAN, DeepLabV3, DeepLabV3+
    encoders_name: str
        ResNet, ResNext, ResNeSt, Res2Ne(X)t, RegNet(x/y), GERNet, SE-Net
        SK-ResNe(X)t, DenseNet, Inception, EfficientNet, MobileNet, DPN, VGG
    encoder_depth: int [3,5]
        downsampling times
    encoder_weights: Optional: 'imagenet' or None
        pretrained weights
    decoder_channels: list
        same length as encoder_depth
    decoder_use_batchnorm: bool or 'inplace'
        BatchNorm2d between Conv2D and Activation layer will be implemented
    decoder_attention_type: Optional: 'scse' or None
        Attention module used in decoder
    in_channels: int
        input images channels
    classes: int 
        output number of classes
    activation: str
        sigmoid, softmax, logsoftmax, tanh, identity, callable or None
    aux_params: dict or None
        auxiliary output build on top of encoder: 
            {classes (int), pooling (str), dropout: [0,1), avtivation (str)}
    '''
    model_params = {
        'arch': cfg.model, 
        'encoder_name': cfg.encoder_name,
        'encoder_depth': cfg.encoder_depth,
        'encoder_weights': cfg.encoder_weights,        
        'in_channels': cfg.inchannel,
        'classes': cfg.num_classes,
        'activation': cfg.activation,
        'aux_params': cfg.aux_params,    
    }

    if cfg.model == 'Unet' or cfg.model == 'Unet++':

        model_params['decoder_channels'] = cfg.decoder_channels
        model_params['decoder_use_batchnorm'] = cfg.decoder_use_batchnorm
        model_params['decoder_attention_type'] = cfg.decoder_attention_type

    if cfg.model == 'MAnet':

        model_params['decoder_channels'] = cfg.decoder_channels
        model_params['decoder_use_batchnorm'] = cfg.decoder_use_batchnorm
        model_params['decoder_pab_channels'] = cfg.decoder_pab_channels
    
    if cfg.model == 'Linknet':

        model_params['decoder_use_batchnorm'] = cfg.decoder_use_batchnorm

    if cfg.model == 'FPN':

        model_params['decoder_pyramid_channels'] = cfg.decoder_pyramid_channels
        model_params['decoder_segmentation_channels'] = cfg.decoder_segmentation_channels
        model_params['decoder_merge_policy'] = cfg.decoder_merge_policy
        model_params['decoder_dropout'] = cfg.decoder_dropout
        model_params['upsampling'] = cfg.upsampling

    if cfg.model == 'PSPNet':

        model_params['psp_out_channels'] = cfg.psp_out_channels
        model_params['psp_use_batchnorm'] = cfg.psp_use_batchnorm
        model_params['psp_dropout'] = cfg.psp_dropout
        model_params['upsampling'] = cfg.upsampling

    if cfg.model == 'PAN':

        model_params['decoder_channels'] = cfg.decoder_channels
        model_params['upsampling'] = cfg.upsampling
    
    if cfg.model == 'DeepLabV3':

        model_params['decoder_channels'] = cfg.decoder_channels
        model_params['upsampling'] = cfg.upsampling

    if cfg.model == 'DeepLabV3+':

        model_params['encoder_output_stride'] = cfg.encoder_output_stridee
        model_params['decoder_atrous_rates'] = cfg.decoder_atrous_rates
        model_params['decoder_channels'] = cfg.decoder_channels
        model_params['upsampling'] = cfg.upsampling

    return model_params