from torch.optim import Adam, SGD, AdamW
from torch.optim.lr_scheduler import StepLR, MultiStepLR, ExponentialLR, CosineAnnealingLR, ReduceLROnPlateau

def optims(model, config):

    if config.optimizer == 'Adam':
        return Adam(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)

    if config.optimizer == 'SGD':
        return SGD(model.parameters(), lr=config.lr, momentum=config.momentum, weight_decay=config.weight_decay)

    if config.optimizer == 'AdamW':
        return AdamW(model.parameters(), lr=config.lr, weight_decay=config.weight_decay)

    return None


def scheduler(optimizer, config):

    if config.lr_scheduler == 'StepLR':
        return StepLR(optimizer, step_size=config.lr_step_size, gamma=config.lr_gamma)

    if config.lr_scheduler == 'MultiStepLR':
        return MultiStepLR(optimizer, milestones=config.lr_milestones, gamma=config.lr_gamma)

    if config.lr_scheduler == 'ExponentialLR':
        return ExponentialLR(optimizer, gamma=config.lr_gamma)

    if config.lr_scheduler == 'CosineAnnealingLR':
        return CosineAnnealingLR(optimizer, T_max=config.lr_T_max, eta_min=config.lr_eta_min)

    if config.lr_scheduler == 'ReduceLROnPlateau':
        return ReduceLROnPlateau(optimizer, mode=config.lr_reduce_mode, factor=config.lr_reduce_factor, patience=config.lr_reduce_patience, \
                                threshold=config.lr_reduce_threshold, cooldown=config.lr_reduce_cooldown, min_lr=config.lr_reduce_min_lr)

    return None